from typing import List, Tuple, Dict, AnyStr
from pathlib import Path
from . import utils


class EnvBase:

    __slots__ = (
        "env_src",
        "env_lines",
    )

    def __iter__(self):
        for line in (line for line in self.env_lines if line and "=" in line):
            yield line.split("=", 1)

    @property
    def kv_pairs(self) -> List[Tuple[str, str]]:
        return [kv for kv in self]

    @property
    def keys(self) -> List[str]:
        return [key for (key, _) in self]

    @property
    def values(self) -> List[str]:
        return [value for (_, value) in self]

    def dict(self) -> Dict[str, AnyStr]:
        return {key: value for (key, value) in self}


class DotEnv(EnvBase):
    def __init__(self, path: str):
        env_src = Path(path).read_text()
        env_lines = [
            line for line in env_src.splitlines(keepends=False) if line and "=" in line
        ]
        self.env_src = env_src
        self.env_lines = env_lines


class PythonEnv(EnvBase):
    pass


def docker_compose_env_lines(__docker_compose_src: str):
    indent, results = None, []
    for line in __docker_compose_src.splitlines(keepends=False):
        line_indent = utils.indent(line)
        if indent is None:
            if line.strip().endswith("environment:"):
                indent = line_indent
        elif line_indent > indent:
            ffl = utils.from_first_letter(line)
            if "=" in ffl:
                results.append(ffl)
        else:
            indent = None
    return results


class DockerEnv(EnvBase):
    def __init__(self, path: str):
        self.env_src = Path(path).read_text()
        self.env_lines = docker_compose_env_lines(self.env_src)



from .objects import DotEnv

dot_env_keys = lambda dotenv_path: DotEnv(dotenv_path)
env_fields_src = lambda env_keys: "\n".join(
    map(lambda key: f"    {key}:Optional[str] = Field(default=None)", env_keys)
)
env_init_src = (
    "env = Env(**{" + "key:os.environ.get(key) for key in Env.__fields__.keys()})\n"
)
python_env_module_src = (
    lambda dotenv_path: f"""
import os
from typing import Optional
from dotenv import load_dotenv
from pydantic import BaseModel, Field

load_dotenv('{dotenv_path}')

class Env(BaseModel):

{env_fields_src(DotEnv(dotenv_path).keys)}

{env_init_src}
"""
)



from . import objects as __objects
from . import templates as __templates
from pathlib import Path as __Path


def __docker_compose_env_dict(docker_compose_path: str):
    return __objects.DockerEnv(docker_compose_path).dict()


def __dotenv_dict(dotenv_path: str):
    return __objects.DotEnv(dotenv_path).dict()


def __form_dotenv_source(__env_dict: dict):
    env_lines = [f"{key}={value}" for key, value in __env_dict.items()]
    env_lines.sort()
    env_source = "\n".join(env_lines) if env_lines else ""
    return env_source


def __write_dotenv_from_docker_compose(dotenv_path: str, docker_compose_path: str):
    env_dict = __docker_compose_env_dict(docker_compose_path)
    source = __form_dotenv_source(env_dict)
    __Path(dotenv_path).write_text(source)


def __write_python_env_module_from_dotenv(
    python_env_module_path: str, dotenv_path: str
):
    python_env_module_source = __templates.python_env_module_src(dotenv_path)
    __Path(python_env_module_path).write_text(python_env_module_source)


def docker_compose_env_dict(docker_compose_path: str):
    return __docker_compose_env_dict(docker_compose_path)


def dotenv_dict(dotenv_path: str):
    return __dotenv_dict(dotenv_path)


def refresh_env_from_docker_compose(
    docker_compose_path: str = "docker-compose.yml",
    dotenv_path: str = ".env",
    python_env_module_path: str = "env.py"
):
    __write_dotenv_from_docker_compose(dotenv_path, docker_compose_path)
    __write_python_env_module_from_dotenv(python_env_module_path, dotenv_path)


